# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

# -- Project information -----------------------------------------------------

import os
import sys
from pathlib import Path


project = u"xrt_spec_dl"
copyright = u"2021, J. Michael Burgess"
author = u"J. Michael Burgess"


sys.path.insert(0, os.path.abspath("../"))

DOCS = Path(__file__).parent


# -- Generate API documentation ------------------------------------------------
def run_apidoc(app):
    """Generage API documentation"""
    import better_apidoc

    better_apidoc.APP = app
    better_apidoc.main(
        [
            "better-apidoc",
            # "-t",
            # str(docs / "_templates"),
            "--force",
            "--no-toc",
            "--separate",
            "-o",
            str(DOCS / "api"),
            str(DOCS / ".." / "xrt_spec_dl"),
        ]
    )


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "nbsphinx",
    "recommonmark",
    "sphinx.ext.autodoc",
    "sphinx.ext.mathjax",
    "sphinx.ext.viewcode",
    "sphinx.ext.autodoc",
    "sphinx.ext.githubpages",
    "sphinx.ext.napoleon",
    #    'rtds_action'
]


napoleon_google_docstring = True
napoleon_use_param = True
napoleon_include_private_with_doc = True
napoleon_include_init_with_doc = True

autodoc_default_options = {
    "members": "var1, var2",
    "member-order": "bysource",
    "special-members": "__init__",
    "undoc-members": True,
    "exclude-members": "__weakref__",
}


# # The name of your GitHub repository
# rtds_action_github_repo = "grburgess/xrt_spec_dl"

# # The path where the artifact should be extracted
# # Note: this is relative to the conf.py file!
# rtds_action_path = "notebooks"

# # The "prefix" used in the `upload-artifact` step of the action
# rtds_action_artifact_prefix = "notebooks-for-"

# # A GitHub personal access token is required, more info below
# rtds_action_github_token = os.environ["GITHUB_TOKEN"]


# rtds_action_error_if_missing = True


# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store", "**.ipynb_checkpoints"]


# nbsphinx_allow_errors =True
# nbsphinx_execute = "never"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".

html_static_path = ["_static"]

# These paths are either relative to html_static_path
# or fully qualified paths (eg. https://...)
# html_css_files = [
#     'css/custom.css',
# ]

# html_js_files = [
#     'css/custom.js',
# ]

html_show_sourcelink = False
html_favicon = "media/favicon.ico"

html_show_sphinx = False

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

# autosectionlabel_prefix_document = True

# avoid time-out when running the doc
nbsphinx_timeout = 30 * 60

nbsphinx_execute_arguments = [
    "--InlineBackend.figure_formats={'svg', 'pdf'}",
    "--InlineBackend.rc={'figure.dpi': 96}",
]


from pygments.formatters import HtmlFormatter  # noqa: E402
from pygments.styles import get_all_styles  # noqa: E402

path = os.path.join("_static", "pygments")
if not os.path.isdir(path):
    os.mkdir(path)
for style in get_all_styles():
    path = os.path.join("_static", "pygments", style + ".css")
    if os.path.isfile(path):
        continue
    with open(path, "w") as f:
        f.write(HtmlFormatter(style=style).get_style_defs(".highlight"))

html_theme_options = {
    "logo_only": False,
    "display_version": False,
    "collapse_navigation": True,
    "navigation_depth": 4,
    "prev_next_buttons_location": "bottom",  # top and bottom
}

# Output file base name for HTML help builder.
htmlhelp_basename = "popsynthdoc"

source_suffix = [".rst"]

html_logo = "media/logo.png"

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {}

master_doc = "index"

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        "xrt_spec_dl.tex",
        u"xrt_spec_dl Documentation",
        u"J. Michael Burgess",
        "manual",
    ),
]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        master_doc,
        "xrt_spec_dl",
        u"xrt_spec_dl Documentation",
        [author],
        1,
    )
]

# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "xrt_spec_dl",
        u"xrt_spec_dl Documentation",
        author,
        "xrt_spec_dl",
        "One line description of project.",
        "Miscellaneous",
    ),
]


def setup(app):
    app.connect("builder-inited", run_apidoc)



import logging
import os
import shutil
from glob import glob
from pathlib import Path

import pytest
from xrt_spec_dl.utils.package_utils import get_path_of_data_file



@pytest.fixture(scope="session")
def thing():
    pass



import urllib3
from pathlib import Path
import os
import time
import requests
import tarfile

from .utils.logging import setup_logger
from .utils.file_utils import (
    sanitize_filename,
    if_directory_not_existing_then_make,
)

log = setup_logger(__name__)


_allow_modes = ("BOTH", "PC", "WT")


def download_xrt_spectral_data(
    obs_id: str,
    name: str,
    tstart: float,
    tstop: float,
    mode: str = "BOTH",
    filename: str = "xrt.tar.gz",
    destination_dir: str = ".",
):

    """TODO describe function

    :param obs_id:
    :type obs_id: str
    :param name:
    :type name: str
    :param tstart:
    :type tstart: float
    :param tstop:
    :type tstop: float
    :param mode:
    :type mode: str
    :param filename:
    :type filename: str
    :param destination_dir:
    :type destination_dir: str
    :returns:

    """
    if mode not in _allow_modes:

        log.error(f"{mode} is not one of {','.join(_allow_modes)}")

        raise RuntimeError()

    # set the time slice to analyze
    timeslices: str = f"{tstart}-{tstop}"

    destination: Path = sanitize_filename(destination_dir)

    if_directory_not_existing_then_make(destination)

    # build the form
    data = dict(
        targ=obs_id,
        name=name,
        pubpriv=1,
        public=1,
        z=0,
        sno=0,
        name1="a",
        time1=timeslices,
        mode1=mode,
        name2="",
        time2="",
        mode2=mode,
        name3="",
        time3="",
        mode3=mode,
        name4="",
        time4="",
        mode4=mode,
        grade0="0",
    )

    log.info("requesting build...")

    r = requests.post(
        "http://www.swift.ac.uk/xrt_spectra/build_slice_spec.php", params=data
    )

    url = r.url

    r2 = r
    itr = 0

    if "Error" in r2.content.decode():

        print(r2.content.decode())

        raise RuntimeError()

    while "This page will be reloaded in 30 seconds" in r2.content.decode():

        log.info("sleeping 30 seconds ...")

        time.sleep(30)

        if itr == 0:
            log.info(f"requesting {url} ...")

        r2 = requests.get(url)

        itr += 1

    log.info(f"downloading: {os.path.join(url, filename)}")
    connection_pool = urllib3.PoolManager()
    resp = connection_pool.request("GET", os.path.join(url, "a.tar.gz"))
    ""

    final_path: Path = destination / filename

    with final_path.open("wb") as f:
        f.write(resp.data)

    resp.release_conn()

    tar = tarfile.open(final_path)

    tar.extractall(path=destination)

    paths = dict()

    pc = (
        dict(
            source=os.path.join(destination_dir, "apcsource.pi"),
            bak=os.path.join(destination_dir, "apcbak.pi"),
            rmf=os.path.join(destination_dir, "apc.rmf"),
            arf=os.path.join(destination_dir, "apc.arf"),
        ),
    )

    wt = dict(
        source=os.path.join(destination_dir, "awtsource.pi"),
        bak=os.path.join(destination_dir, "awtbak.pi"),
        rmf=os.path.join(destination_dir, "awt.rmf"),
        arf=os.path.join(destination_dir, "awt.arf"),
    )

    if mode == "BOTH":

        paths["PC"] = pc
        paths["WT"] = wt

    elif mode == "PC":

        paths["PC"] = pc

    else:

        paths["WT"] = wt

    return paths



# -*- coding: utf-8 -*-

from .xrt_spec_dl import download_xrt_spectral_data


"""Top-level package for xrt_spec_dl."""

__author__ = """J. Michael Burgess"""
__email__ = "jburgess@mpe.mpg.de"

from . import _version
__version__ = _version.get_versions()['version']
